@@ -36,6 +36,7 @@ mod c2rust;
 mod ecdh;
 mod kdf;
 mod mac;
+mod provisioning;
 mod rust2c;
 
 struct AteccResourceManager {
@@ -264,6 +265,37 @@ impl AteccDeviceTrait for AteccDevice {
         self.ecdh(parameters, peer_public_key)
     } // AteccDevice::ecdh()
 
+    /// Execute this command prevents future modifications of the Configuration zone.
+    /// This command fails if the designated area is already locked.
+    /// Trait implementation
+    fn lock_config_zone(&self) -> AtcaStatus {
+        self.lock_config_zone()
+    } // AteccDevice::lock_config_zone()
+
+    /// Execute this command prevents future modifications of the Data and OTP zones.
+    /// This command fails if the designated area is already locked.
+    /// Trait implementation
+    fn lock_data_zone(&self) -> AtcaStatus {
+        self.lock_data_zone()
+    } // AteccDevice::lock_data_zone()
+
+    /// Lock an individual slot in the data zone on an ATECC device. Not available for ATSHA devices.
+    /// Slot must be configured to be slot lockable slots[slot_idx].config.lockable = true.
+    /// This command fails if the designated area is already locked.
+    /// Trait implementation
+    fn lock_slot(&self, slot_id: u8) -> AtcaStatus {
+        self.lock_slot(slot_id)
+    } // AteccDevice::lock_slot()
+
+    /// Function for uploading configuration to the chip.
+    /// First 16 bytes of data are skipped as they are not writable. LockValue and LockConfig
+    /// are also skipped and can only be changed via the Lock command.
+    /// This command may fail if UserExtra and/or Selector bytes have already been set to non-zero values.
+    /// Trait implementation
+    fn load_config_into_chip(&self, config: &[u8]) -> AtcaStatus {
+        self.load_config_into_chip(config)
+    } // AteccDevice::load_config_into_chip()
+
     /// Request ATECC to return own device type
     /// Trait implementation
     fn get_device_type(&self) -> AtcaDeviceType {
