@@ -59,6 +59,7 @@ pub struct SignEcdsaParam {
 }
 
 /// Detailed parameters of calling the ECDSA verification function
+#[derive(Default)]
 pub struct VerifyEcdsaParam {
     /// Public key for ExternalMac mode 
     pub public_key: Option<Vec<u8>>,
@@ -70,17 +71,6 @@ pub struct VerifyEcdsaParam {
     pub io_key: u8,
 }
 
-impl Default for VerifyEcdsaParam {
-    fn default() -> VerifyEcdsaParam {
-        VerifyEcdsaParam {
-            public_key: None,
-            slot_number: None,
-            num_in: Vec::new(),
-            io_key: 0x00,
-        }
-    }
-}
-
 /// Cipher operation type
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub enum CipherOperation {
@@ -114,7 +104,7 @@ pub enum CipherAlgorithm {
 }
 
 /// Cipher algorithm parameters for compute
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, PartialEq, Default)]
 pub struct CipherParam {
     /// IV - Initialization Vector.
     /// For CTR mode it is concatenation of nonce and initial counter value.
@@ -126,16 +116,6 @@ pub struct CipherParam {
     pub key: Option<Vec<u8>>,
 }
 
-impl Default for CipherParam {
-    fn default() -> CipherParam {
-        CipherParam {
-            iv: None,
-            counter_size: None,
-            key: None,
-        }
-    }
-}
-
 /// Type of AEAD algorithm
 #[derive(Clone, Debug, PartialEq)]
 pub enum AeadAlgorithm {
@@ -144,7 +124,7 @@ pub enum AeadAlgorithm {
 }
 
 /// AEAD algorithm parameters for compute
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, PartialEq, Default)]
 pub struct AeadParam {
     /// Nonce [number used once aka IV] (default length is 12 bytes)
     pub nonce: Vec<u8>,
@@ -159,18 +139,6 @@ pub struct AeadParam {
     pub additional_data: Option<Vec<u8>>,
 }
 
-impl Default for AeadParam {
-    fn default() -> AeadParam {
-        AeadParam {
-            nonce: Vec::new(),
-            key: None,
-            tag: None,
-            tag_length: None,
-            additional_data: None,
-        }
-    }
-}
-
 /// MAC algorithm to compute
 #[derive(Clone, Debug, PartialEq)]
 pub enum MacAlgorithm {
@@ -180,7 +148,7 @@ pub enum MacAlgorithm {
 }
 
 /// MAC algorithm parameters for compute
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, PartialEq, Default)]
 pub struct MacParam {
     /// external encryption/decryption key needed for MAC calculation
     /// when an 'AES' or 'ShaOrText' key stored in the cryptochip is not used
@@ -191,16 +159,220 @@ pub struct MacParam {
     pub mac: Option<Vec<u8>>,
 }
 
-impl Default for MacParam {
-    fn default() -> MacParam {
-        MacParam {
-            key: None,
-            mac_length: None,
-            mac: None,
+/// KDF algorithm to derive key
+#[derive(Clone, Debug, PartialEq)]
+pub enum KdfAlgorithm {
+    Prf(PrfDetails),
+    Hkdf(HkdfDetails),
+    Aes,
+}
+
+/// KDF function parameters
+#[derive(Clone, Debug, PartialEq)]
+pub struct KdfParams {
+    pub source: KdfSource,
+    pub target: KdfTarget,
+    pub source_slot_id: Option<u8>,
+    pub target_slot_id: Option<u8>,
+}
+
+impl Default for KdfParams {
+    fn default() -> KdfParams {
+        KdfParams {
+            source: KdfSource::TempKey,
+            target: KdfTarget::Output,
+            source_slot_id: None,
+            target_slot_id: None,
         }
     }
 }
 
+/// KDF sources
+#[derive(Clone, Debug, PartialEq)]
+#[repr(u8)]
+pub enum KdfSource {
+    /// source key in TempKey
+    TempKey = 0x00,
+    /// source key in upper TempKey
+    TempKeyUp = 0x01,
+    /// source key in a slot
+    Slot = 0x02,
+    /// source key in alternate key buffer
+    AltKeyBuf = 0x03,
+}
+
+/// KDF targets. Possibility of exporting KDF function result outside the chip
+/// depends on "chip_options.kdf_output_protection" variable
+#[derive(Clone, Debug, PartialEq)]
+#[repr(u8)]
+pub enum KdfTarget {
+    /// target key in TempKey
+    TempKey = 0x00,
+    /// target key in upper TempKey
+    TempKeyUp = 0x04,
+    /// target key in slot
+    Slot = 0x08,
+    /// target key in alternate key buffer
+    AltKeyBuf = 0x0C,
+    /// target key in output buffer
+    Output = 0x10,
+    /// target key encrypted in output buffer
+    OutputEnc = 0x14,
+}
+
+/// KDF details for PRF, source key length
+#[derive(Clone, Debug, PartialEq)]
+#[repr(u32)]
+pub enum KdfPrfKeyLen {
+    /// source key length is 16 bytes
+    Len16 = 0x00000000,
+    /// source key length is 32 bytes
+    Len32 = 0x00000001,
+    /// source key length is 48 bytes
+    Len48 = 0x00000002,
+    /// source key length is 64 bytes
+    Len64 = 0x00000003,
+}
+
+/// KDF details for PRF, target length
+#[derive(Clone, Debug, PartialEq)]
+#[repr(u32)]
+pub enum KdfPrfTargetLen {
+    /// target length is 32 bytes
+    Len32 = 0x00000000,
+    /// target length is 64 bytes
+    Len64 = 0x00000100,
+}
+
+impl From<KdfPrfTargetLen> for usize {
+    fn from(orig: KdfPrfTargetLen) -> Self {
+        match orig {
+            KdfPrfTargetLen::Len32 => 32,
+            KdfPrfTargetLen::Len64 => 64,
+        }
+    }
+}
+
+/// KDF details for PRF
+#[derive(Clone, Debug, PartialEq)]
+pub struct PrfDetails {
+    pub key_length: KdfPrfKeyLen,
+    pub target_length: KdfPrfTargetLen,
+}
+
+impl Default for PrfDetails {
+    fn default() -> PrfDetails {
+        PrfDetails {
+            key_length: KdfPrfKeyLen::Len32,
+            target_length: KdfPrfTargetLen::Len64,
+        }
+    }
+}
+
+/// KDF details for HKDF. [place from which function should retrieve message for calculations]
+#[derive(Clone, Debug, PartialEq)]
+#[repr(u32)]
+pub enum HkdfMsgLoc {
+    /// message location in slot
+    Slot = 0x00000000,
+    /// message location in TempKey
+    TempKey = 0x00000001,
+    /// message location in input parameter
+    Input = 0x00000002,
+    /// message location is a special IV function
+    Iv = 0x00000003,
+}
+
+/// KDF details for HKDF
+#[derive(Clone, Debug, PartialEq)]
+pub struct HkdfDetails {
+    pub msg_loc: HkdfMsgLoc,
+    /// if true then a vector of thirty-two zero bytes will be used as the key
+    pub zero_key: bool,
+    /// if source of message is a slot, its identifier must be entered
+    pub msg_slot: Option<u8>,
+}
+
+impl Default for HkdfDetails {
+    fn default() -> HkdfDetails {
+        HkdfDetails {
+            msg_loc: HkdfMsgLoc::Input,
+            zero_key: false,
+            msg_slot: None,
+        }
+    }
+}
+
+/// KDF result structure
+#[derive(Clone, Debug, PartialEq)]
+pub struct KdfResult {
+    /// Data are available only when the target of the KDF function is 'Output' or 'OutputEnc'
+    pub out_data: Option<Vec<u8>>,
+    /// Data are available only when the target of the KDF function is 'OutputEnc'
+    pub out_nonce: Option<Vec<u8>>,
+}
+
+/// ECDH function parameters
+#[derive(Clone, Debug, PartialEq)]
+pub struct EcdhParams {
+    /// private key source for ECDH
+    pub key_source: EcdhSource,
+    /// target where the result of the ECDH operation will be placed
+    pub out_target: EcdhTarget,
+    /// parameter that specifies whether the output should be encrypted
+    /// (only relevant for the ATECC608x chip)
+    pub out_encrypt: bool,
+    /// slot number where the private key will be retrieved
+    /// or where result of the ECDH operation will be placed
+    pub slot_id: Option<u8>,
+}
+
+impl Default for EcdhParams {
+    fn default() -> EcdhParams {
+        EcdhParams {
+            key_source: EcdhSource::Slot,
+            out_target: EcdhTarget::Compatibility,
+            out_encrypt: false,
+            slot_id: None,
+        }
+    }
+}
+
+/// Private key source for ECDH
+#[derive(Clone, Debug, PartialEq)]
+#[repr(u8)]
+pub enum EcdhSource{
+    /// source key in a slot
+    Slot = 0x00,
+    /// source key in TempKey (only relevant for the ATECC608x chip)
+    TempKey = 0x01,
+}
+
+/// Target where the result of the ECDH operation will be placed
+#[derive(Clone, Debug, PartialEq)]
+#[repr(u8)]
+pub enum EcdhTarget {
+    /// Compatibility mode for ATECC508A. Result goes to either the output buffer
+    /// or (slot_id + 1) depending on the state of slots[slot_idx].config.ecc_key_attr.ecdh_secret_out
+    Compatibility = 0x00,
+    /// Result goes to slot specified by KeyID. slots[slot_idx].write_config must be ALWAYS
+    /// (only relevant for the ATECC608x chip)
+    Slot  = 0x04,
+    /// Result goes to TempKey (only relevant for the ATECC608x chip)
+    TempKey = 0x08,
+    /// Result goes to the output buffer (only relevant for the ATECC608x chip)
+    Output = 0x0C,
+}
+
+/// ECDH result structure
+#[derive(Clone, Debug, PartialEq)]
+pub struct EcdhResult {
+    /// Computed ECDH pre-master secret (32 bytes) if returned directly
+    pub pms: Option<Vec<u8>>,
+    /// Nonce used to encrypt pre-master secret
+    pub out_nonce: Option<Vec<u8>>,
+}
+
 /// Data context structure for AEAD encryption in CCM mode
 #[derive(Copy, Clone, Debug)]//, PartialEq)]
 pub struct AtcaAesCcmCtx {
@@ -252,14 +424,22 @@ pub struct ChipOptions {
     pub io_key_enabled: bool,
     /// slot number where the key for encrypting transmission between chip and host is placed
     pub io_key_in_slot: u8,
-    ///
+    /// flag, on-chip availability, AES function
     pub aes_enabled: bool,
-    ///
+    /// flag, on-chip availability, AES functionality for KDF command
     pub kdf_aes_enabled: bool,
-    ///
+    /// restrictions on the way the ECDH command result can be used
     pub ecdh_output_protection: OutputProtectionState,
-    ///
+    /// restrictions on the way the KDF command result can be used
     pub kdf_output_protection: OutputProtectionState,
+    /// availability flag of the special function of the IV KDF command 
+    pub kdf_iv_enabled: bool,
+    /// place in message where special data bytes must be placed
+    /// when calling function IV of the KDF command
+    pub kdf_iv_location_at: usize,
+    /// two bytes of data that must be included in message
+    /// when calling function IV of the KDF command
+    pub kdf_iv_str: [u8; 0x02],
 }
 
 impl Default for ChipOptions {
@@ -271,6 +451,9 @@ impl Default for ChipOptions {
             kdf_aes_enabled: false,
             ecdh_output_protection: OutputProtectionState::Invalid,
             kdf_output_protection: OutputProtectionState::Invalid,
+            kdf_iv_enabled: false,
+            kdf_iv_location_at: 0x00,
+            kdf_iv_str: [0x00, 0x00],
         }
     }
 }
@@ -303,7 +486,7 @@ impl From<u8> for OutputProtectionState {
 }
 
 /// An ATECC slot
-#[derive(Copy, Clone, Debug)]
+#[derive(Copy, Clone, Debug, Default)]
 pub struct AtcaSlot {
     /// ATECC slot id (for diagnostic)
     pub id: u8,
@@ -314,34 +497,14 @@ pub struct AtcaSlot {
     pub config: SlotConfig,
 }
 
-impl Default for AtcaSlot {
-    fn default() -> Self {
-        AtcaSlot {
-            id: 0u8,
-            is_locked: false,
-            config: SlotConfig::default(),
-        }
-    }
-}
-
 /// An ATECC slot capacity
-#[derive(Copy, Clone, Debug)]
+#[derive(Copy, Clone, Debug, Default)]
 pub struct AtcaSlotCapacity {
     pub blocks: u8,
     pub last_block_bytes: u8,
     pub bytes: u16,
 }
 
-impl Default for AtcaSlotCapacity {
-    fn default() -> Self {
-        AtcaSlotCapacity {
-            blocks: 0u8,
-            last_block_bytes: 0u8,
-            bytes: 0u16,
-        }
-    }
-}
-
 /// Detailed ATECC key slot configuration
 #[derive(Copy, Clone, Debug)]
 pub struct SlotConfig {
@@ -468,7 +631,7 @@ impl Default for SlotConfig {
 }
 
 /// Detailed ECC key attributes as stored in slot configuration
-#[derive(Copy, Clone, Debug)]
+#[derive(Copy, Clone, Debug, Default)]
 pub struct EccKeyAttr {
     /// true = The key slot contains an ECC private key and
     /// can be accessed only with the Sign, GenKey, and PrivWrite commands.
@@ -501,20 +664,8 @@ pub struct EccKeyAttr {
     pub ecdh_secret_out: bool,
 }
 
-impl Default for EccKeyAttr {
-    fn default() -> Self {
-        EccKeyAttr {
-            is_private: false,
-            ext_sign: false,
-            int_sign: false,
-            ecdh_operation: false,
-            ecdh_secret_out: false,
-        }
-    }
-}
-
 /// Detailed ATECC key slot read attributes
-#[derive(Copy, Clone, Debug)]
+#[derive(Copy, Clone, Debug, Default)]
 pub struct ReadKey {
     /// true = Reads from this slot will be encrypted using the procedure
     /// specified in the Read command using value of 'slot_number'
@@ -533,15 +684,6 @@ pub struct ReadKey {
     pub slot_number: u8,
 }
 
-impl Default for ReadKey {
-    fn default() -> Self {
-        ReadKey {
-            encrypt_read: false,
-            slot_number: 0u8,
-        }
-    }
-}
-
 /// Detailed ATECC key slot write configuration
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub enum WriteConfig {
@@ -607,7 +749,7 @@ pub union AtcaIface {
 } // pub union AtcaIface
 
 /// ATECC I2C interface details
-#[derive(Copy, Clone)]
+#[derive(Copy, Clone, Default)]
 pub struct AtcaIfaceI2c {
     /// ATECC I2C bus address
     slave_address: u8,
